package me.totalfreedom.totalfreedommod.sql;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Wrapper;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.BiConsumer;
import java.util.function.Consumer;

public class ResultSetProvider
{
    List<Map<String, Object>> allRowsResultSet = new ArrayList<>();

    Map<String, Object> singularResultMap = new HashMap<>();

    private ResultSetProvider(Map<String, Object> singularResultMap)
    {
        this.singularResultMap = singularResultMap;
    }

    private ResultSetProvider(ResultSet resultSet, boolean allRows) throws SQLException
    {
        if (allRows)
        {
            while (resultSet.next())
            {
                allRowsResultSet.add(mapResultSet(resultSet));
            }
        } else
        {
            if (resultSet.next())
            {
                this.singularResultMap = mapResultSet(resultSet);
            }
        }
    }

    public static CompletableFuture<ResultSetProvider> provideAllRows(ResultSet resultSet)
    {
        return CompletableFuture.supplyAsync(() ->
        {
            try
            {
                return new ResultSetProvider(resultSet, true);
            } catch (SQLException e)
            {
                return null;
            }
        });
    }

    public static CompletableFuture<ResultSetProvider> provideSpecificRow(ResultSet resultSet) throws SQLException
    {
        return CompletableFuture.supplyAsync(() -> {
            try
            {
                return new ResultSetProvider(resultSet, false);
            } catch (SQLException e)
            {
                throw new RuntimeException(e); // We don't want to handle this exception, we want the delegate to handle it instead.
            }
        });
    }

    public static ResultSetProvider fromRow(Map<String, Object> row) {
        return new ResultSetProvider(row);
    }

    public List<Map<String, Object>> getAllRowsResultSet()
    {
        return allRowsResultSet;
    }

    private Map<String, Object> mapResultSet(ResultSet rs) throws SQLException
    {
        Map<String, Object> result = new HashMap<>();
        while (rs.next())
        {
            int columns = rs.getMetaData().getColumnCount();
            for (int i = 1; i <= columns; i++)
            {
                String columnName = rs.getMetaData().getColumnName(i);
                Object columnValue = rs.getObject(i);
                result.put(columnName, columnValue);
            }
        }
        return result;
    }

    public String getString(String key)
    {
        return (String) singularResultMap.get(key);
    }

    public int getInt(String key)
    {
        return (int) singularResultMap.get(key);
    }

    public boolean getBoolean(String key)
    {
        return (boolean) singularResultMap.get(key);
    }

    public long getLong(String key)
    {
        return (long) singularResultMap.get(key);
    }
}
